@@ -38,8 +38,8 @@
             template<typename Expr, typename State, typename Data, typename Tag, long Arity>
             struct impl2;
 
-            template<typename Expr, typename State, typename Data>
-            struct impl2<Expr, State, Data, tag::terminal, 0>
+            template<typename Expr, typename State, typename Data, typename Tag>
+            struct impl2<Expr, State, Data, Tag, 0>
               : _value::impl<Expr, State, Data>
             {};
 
@@ -54,13 +54,13 @@
             public:                                                                                 \
                 BOOST_PROTO_DECLTYPE_(OP proto::detail::MAKE<r0>(), result_type)                    \
                 result_type operator ()(                                                            \
-                    typename impl2::expr_param expr                                                 \
-                  , typename impl2::state_param state                                               \
-                  , typename impl2::data_param data                                                 \
+                    typename impl2::expr_param e                                                    \
+                  , typename impl2::state_param s                                                   \
+                  , typename impl2::data_param d                                                    \
                 ) const                                                                             \
                 {                                                                                   \
                     typename Grammar::template impl<e0, State, Data> t0;                            \
-                    return OP t0(proto::child_c<0>(expr), state, data);                             \
+                    return OP t0(proto::child_c<0>(e), s, d);                                       \
                 }                                                                                   \
             };                                                                                      \
             /**/
@@ -81,15 +81,15 @@
                   , result_type                                                                     \
                 )                                                                                   \
                 result_type operator ()(                                                            \
-                    typename impl2::expr_param expr                                                 \
-                  , typename impl2::state_param state                                               \
-                  , typename impl2::data_param data                                                 \
+                    typename impl2::expr_param e                                                    \
+                  , typename impl2::state_param s                                                   \
+                  , typename impl2::data_param d                                                    \
                 ) const                                                                             \
                 {                                                                                   \
                     typename Grammar::template impl<e0, State, Data> t0;                            \
                     typename Grammar::template impl<e1, State, Data> t1;                            \
-                    return t0(proto::child_c<0>(expr), state, data)                                 \
-                        OP t1(proto::child_c<1>(expr), state, data);                                \
+                    return t0(proto::child_c<0>(e), s, d)                                           \
+                        OP t1(proto::child_c<1>(e), s, d);                                          \
                 }                                                                                   \
             };                                                                                      \
             /**/
@@ -161,16 +161,16 @@
             public:
                 typedef typename detail::mem_ptr_fun<r0, r1>::result_type result_type;
                 result_type operator ()(
-                    typename memfun_impl::expr_param expr
-                  , typename memfun_impl::state_param state
-                  , typename memfun_impl::data_param data
+                    typename memfun_impl::expr_param e
+                  , typename memfun_impl::state_param s
+                  , typename memfun_impl::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
                     typename Grammar::template impl<e1, State, Data> t1;
                     return detail::mem_ptr_fun<r0, r1>()(
-                        t0(proto::child_c<0>(expr), state, data)
-                      , t1(proto::child_c<1>(expr), state, data)
+                        t0(proto::child_c<0>(e), s, d)
+                      , t1(proto::child_c<1>(e), s, d)
                     );
                 }
             };
@@ -188,16 +188,16 @@
             public:
                 typedef detail::memfun<r0, r1> result_type;
                 result_type const operator ()(
-                    typename memfun_impl::expr_param expr
-                  , typename memfun_impl::state_param state
-                  , typename memfun_impl::data_param data
+                    typename memfun_impl::expr_param e
+                  , typename memfun_impl::state_param s
+                  , typename memfun_impl::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
                     typename Grammar::template impl<e1, State, Data> t1;
                     return detail::memfun<r0, r1>(
-                        t0(proto::child_c<0>(expr), state, data)
-                      , t1(proto::child_c<1>(expr), state, data)
+                        t0(proto::child_c<0>(e), s, d)
+                      , t1(proto::child_c<1>(e), s, d)
                     );
                 }
             };
@@ -217,13 +217,13 @@
             public:
                 BOOST_PROTO_DECLTYPE_(proto::detail::make_mutable<r0>() ++, result_type)
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
-                    return t0(proto::child_c<0>(expr), state, data) ++;
+                    return t0(proto::child_c<0>(e), s, d) ++;
                 }
             };
 
@@ -237,13 +237,13 @@
             public:
                 BOOST_PROTO_DECLTYPE_(proto::detail::make_mutable<r0>() --, result_type)
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
-                    return t0(proto::child_c<0>(expr), state, data) --;
+                    return t0(proto::child_c<0>(e), s, d) --;
                 }
             };
 
@@ -262,15 +262,15 @@
                   , result_type
                 )
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
                     typename Grammar::template impl<e1, State, Data> t1;
-                    return t0(proto::child_c<0>(expr), state, data) [
-                           t1(proto::child_c<1>(expr), state, data) ];
+                    return t0(proto::child_c<0>(e), s, d) [
+                           t1(proto::child_c<1>(e), s, d) ];
                 }
             };
 
@@ -293,17 +293,17 @@
                   , result_type
                 )
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
                     typename Grammar::template impl<e1, State, Data> t1;
                     typename Grammar::template impl<e2, State, Data> t2;
-                    return t0(proto::child_c<0>(expr), state, data)
-                         ? t1(proto::child_c<1>(expr), state, data)
-                         : t2(proto::child_c<2>(expr), state, data);
+                    return t0(proto::child_c<0>(e), s, d)
+                         ? t1(proto::child_c<1>(e), s, d)
+                         : t2(proto::child_c<2>(e), s, d);
                 }
             };
 
@@ -319,15 +319,15 @@
             public:
                 typedef typename proto::detail::comma_result<r0, r1>::type result_type;
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
                     typename Grammar::template impl<e0, State, Data> t0;
                     typename Grammar::template impl<e1, State, Data> t1;
-                    return t0(proto::child_c<0>(expr), state, data)
-                         , t1(proto::child_c<1>(expr), state, data);
+                    return t0(proto::child_c<0>(e), s, d)
+                         , t1(proto::child_c<1>(e), s, d);
                 }
             };
 
@@ -342,7 +342,7 @@
 
             #define EVAL(Z, N, DATA)                                                                \
                 typename Grammar::template impl<BOOST_PP_CAT(e, N), State, Data>()(                 \
-                    proto::child_c<N>(DATA), state, data                                            \
+                    proto::child_c<N>(DATA), s, d                                            \
                 )                                                                                   \
                 /**/
 
@@ -361,12 +361,12 @@
                 result_type;
 
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
-                    return EVAL(~, 0, expr)();
+                    return EVAL(~, 0, e)();
                 }
             };
 
@@ -386,15 +386,15 @@
                 result_type;
 
                 result_type operator ()(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                 ) const
                 {
                     return this->invoke(
-                        expr
-                      , state
-                      , data
+                        e
+                      , s
+                      , d
                       , is_member_function_pointer<function_type>()
                       , is_member_object_pointer<function_type>()
                     );
@@ -402,38 +402,38 @@
 
             private:
                 result_type invoke(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                   , mpl::false_
                   , mpl::false_
                 ) const
                 {
-                    return EVAL(~, 0, expr)(EVAL(~, 1, expr));
+                    return EVAL(~, 0, e)(EVAL(~, 1, e));
                 }
 
                 result_type invoke(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                   , mpl::true_
                   , mpl::false_
                 ) const
                 {
                     using namespace detail::get_pointer_;
-                    return (get_pointer(EVAL(~, 1, expr)) ->* EVAL(~, 0, expr))();
+                    return (get_pointer(EVAL(~, 1, e)) ->* EVAL(~, 0, e))();
                 }
 
                 result_type invoke(
-                    typename impl2::expr_param expr
-                  , typename impl2::state_param state
-                  , typename impl2::data_param data
+                    typename impl2::expr_param e
+                  , typename impl2::state_param s
+                  , typename impl2::data_param d
                   , mpl::false_
                   , mpl::true_
                 ) const
                 {
                     using namespace detail::get_pointer_;
-                    return (get_pointer(EVAL(~, 1, expr)) ->* EVAL(~, 0, expr));
+                    return (get_pointer(EVAL(~, 1, e)) ->* EVAL(~, 0, e));
                 }
             };
 
@@ -450,7 +450,7 @@
                   , State
                   , Data
                   , typename transform_impl<Expr, State, Data>::expr::proto_tag
-                  , transform_impl<Expr, State, Data>::expr::proto_arity::value
+                  , transform_impl<Expr, State, Data>::expr::proto_arity_c
                 >
             {};
         };
@@ -485,36 +485,36 @@
             result_type;
 
             result_type operator ()(
-                typename impl2::expr_param expr
-              , typename impl2::state_param state
-              , typename impl2::data_param data
+                typename impl2::expr_param e
+              , typename impl2::state_param s
+              , typename impl2::data_param d
             ) const
             {
-                return this->invoke(expr, state, data, is_member_function_pointer<function_type>());
+                return this->invoke(e, s, d, is_member_function_pointer<function_type>());
             }
 
         private:
             result_type invoke(
-                typename impl2::expr_param expr
-              , typename impl2::state_param state
-              , typename impl2::data_param data
+                typename impl2::expr_param e
+              , typename impl2::state_param s
+              , typename impl2::data_param d
               , mpl::false_
             ) const
             {
-                return EVAL(~, 0, expr)(BOOST_PP_ENUM_SHIFTED(N, EVAL, expr));
+                return EVAL(~, 0, e)(BOOST_PP_ENUM_SHIFTED(N, EVAL, e));
             }
 
             result_type invoke(
-                typename impl2::expr_param expr
-              , typename impl2::state_param state
-              , typename impl2::data_param data
+                typename impl2::expr_param e
+              , typename impl2::state_param s
+              , typename impl2::data_param d
               , mpl::true_
             ) const
             {
-                #define M0(Z, M, expr) BOOST_PP_COMMA_IF(BOOST_PP_SUB(M, 2)) EVAL(Z, M, expr)
+                #define M0(Z, M, e) BOOST_PP_COMMA_IF(BOOST_PP_SUB(M, 2)) EVAL(Z, M, e)
                 using namespace detail::get_pointer_;
-                return (get_pointer(EVAL(~, 1, expr)) ->* EVAL(~, 0, expr))(
-                    BOOST_PP_REPEAT_FROM_TO(2, N, M0, expr)
+                return (get_pointer(EVAL(~, 1, e)) ->* EVAL(~, 0, e))(
+                    BOOST_PP_REPEAT_FROM_TO(2, N, M0, e)
                 );
                 #undef M0
             }
